/*
 * File: whdgd.c
 *
 * Author: David Kao, Decision
 *
 * Date: 97/03/03, created
 *
 * It is a sample program to demonstrate how to invoke 
 * the Decision Watch Dog Driver.
 */
/*
 * History
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>


/*
 * new types
 */
#define FALSE	0x00
#define TRUE	0x01


/*
 * constants
 */

#define WATCH_DOG_DEV	"/dev/whdg"
#define WATCH_DOG_LOG	"/etc/whdgd.pid"

#define WATCH_DOG_SYS	1
#define WATCH_DOG_APP	2

/*
 * commands to whdgioctl()
 */
#ifndef MIOC
#define MIOC		('M' << 8)
#endif /* MIOC */

#define MCPOWERON	(MIOC | 0xF8)
#define MCPOWEROFF	(MIOC | 0xF9)
#define MCSETCOUNTER	(MIOC | 0xFA)
#define MCGETCOUNTER	(MIOC | 0xFB)


/*
 * global data strcutures
 */
int   whdg_h;
pid_t whdg_pid;
int   whdg_signo = 0;


/*
 * forward referneces
 */
boolean_t whdg_start();
void	  whdg_sys_standby();
void	  whdg_app_standby();
boolean_t whdg_poweron();
boolean_t whdg_poweroff();
boolean_t whdg_getcounter();
boolean_t whdg_setcounter();
void      whdg_signal();
boolean_t whdg_stop();

char *get_fn();


/*
 * Routines
 */

void
main(argc, argv)
    int  argc;
    char *argv[];
{
    char  *fn;
    int   usage = WATCH_DOG_APP;
    char  c;
    u_int counter, i;

    fn = get_fn(argc, argv);

    /*
     * Starting watch dog facility
     */
    if (!whdg_start(fn, usage)) exit(1);

    switch (usage) {
	case WATCH_DOG_SYS:
	    whdg_sys_standby();
	    break;
	case WATCH_DOG_APP:
	    whdg_app_standby();
	    break;
	default:
	    exit(1);
	    break;
    }
       
    /*
     * Stop watch dog facility
     */
    if (!whdg_stop()) exit(1);

    exit(0);
} /* main() */

boolean_t
whdg_start(fn, usage)
    char *fn;
    int  usage;
{
    int  h;
    char s[32];
    int  flag;

    whdg_pid = getpid();
    sprintf(s, "%d", whdg_pid); 

    fn = (fn == NULL) ? WATCH_DOG_LOG : fn;

    if ((h = open(fn, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR))
        == -1) {
	printf("It's failed to open log file(%s) for Watch Dog\n", WATCH_DOG_LOG);
	return FALSE;
    }
    if (write(h, s, strlen(s)) == -1) {
	printf("It's failed to record information to log file(%s) for Watch Dog\n"
	     , WATCH_DOG_LOG);
	return FALSE;
    }
    if (close(h) == -1) {
	printf("It's failed to close log file(%s) for Watch Dog\n",
								WATCH_DOG_LOG);
	return FALSE;
    }

    if (signal(SIGTERM, whdg_signal) == SIG_ERR) {
	printf("It is failed to register SIGTERM");
	return FALSE;
    } 

    switch (usage) {
	case WATCH_DOG_SYS:
	    flag = O_WRONLY;
	    break;
	case WATCH_DOG_APP:
	    flag = O_RDWR;
	    break;
	default:
	    return FALSE;
    }

    if ((whdg_h = open(WATCH_DOG_DEV, flag)) == -1) {
	printf("There is no Watch Dog device %s\n", WATCH_DOG_DEV);
	return FALSE;
    }

    return TRUE;
} /* whdg_start() */

void
whdg_sys_standby()
{
    do {
	pause();
    } while (whdg_signo != SIGTERM);

    return;
} /* whdg_sys_standby() */

void
whdg_app_standby()
{
    int  nsecs = 7;
    char c;

    do {
	/*
	 * notify the watch dog driver that I am alive
	 */
	read(whdg_h, &c, 1);

	/*
	 * doing major routines
	 */
  
	/*
	 * sleep for a while
	 */
	printf("whdg_app_standby(): sleep %d sec...\n", nsecs); 
	sleep(nsecs);
    } while (whdg_signo != SIGTERM);
} /* whdg_app_standby() */

boolean_t
whdg_poweron()
{
    if (ioctl(whdg_h, MCPOWERON, NULL) == -1) {
	printf("It is failed to ioctl the Watch Dog\n");

	return FALSE;
    }

    return TRUE;
} /* whdg_poweron() */

boolean_t
whdg_poweroff()
{
    if (ioctl(whdg_h, MCPOWEROFF, NULL) == -1) {
	printf("It is failed to ioctl the Watch Dog\n");

	return FALSE;
    }

    return TRUE;
} /* whdg_poweroff() */

boolean_t
whdg_getcounter(u_int *counter)
{
    if (ioctl(whdg_h, MCGETCOUNTER, counter) == -1) {
	printf("It is failed to ioctl the Watch Dog\n");

	return FALSE;
    }

    return TRUE;
} /* whdg_getcounter() */

boolean_t
whdg_setcounter(u_int counter)
{
    if (ioctl(whdg_h, MCSETCOUNTER, &counter) == -1) {
	printf("It is failed to ioctl the Watch Dog\n");

	return FALSE;
    }

    return TRUE;
} /* whdg_setcounter() */

void
whdg_signal(signo)
    int signo;
{
    whdg_signo = signo;

    return;
} /* whdg_signal() */

boolean_t
whdg_stop()
{
    if (close(whdg_h) == -1) {
	printf("It is failed to stop Watch Dog\n");

	return FALSE; 
    }
   
    return TRUE;
} /* whdg_stop() */

char*
get_fn(argc, argv)
    int  argc;
    char *argv[];
{
    if (argc < 2) return NULL;

    if (argc != 2) return NULL;

    return argv[1]; 
} /* get_fn() */
